//
import { Component, html, css, raw } from 'wkit'
import { object2query } from '../utils.js'
import { __ROUTER_VIEW__ } from '../constants.js'

class RouterView extends Component {
  static props = {
    transition: false,
    current: {
      type: String,
      default: '',
      attribute: false,
      observer(v, old) {
        if (this.keepAlive && v) {
          if (old && this.$refs[old]) {
            this.$refs[old].deactivated()
          }
          this.$refs[v]?.$requestUpdate()
          this.$refs[v]?.$animate()
          this.$refs[v]?.activated()
        }
      }
    }
  }

  static styles = css`
    :host {
      display: block;
    }
  `

  #views = []

  created() {
    window.wkitd.set(__ROUTER_VIEW__, this)
  }

  sync(views) {
    this.#views = views
  }

  render() {
    let option = {
      immediate: true,
      custom: [
        { transform: 'translateX(-32px)', opacity: 0 },
        { transform: 'translateX(0)', opacity: 1 }
      ]
    }
    if (this.keepAlive) {
      let template = this.#views.map(it =>
        this.transition
          ? [
              `<${it} ref="${it}" keep-alive #animation="%s" style="%s"></${it}>`,
              [
                { ...option, immediate: this.current === it },
                this.current === it ? '' : 'display:none'
              ]
            ]
          : [
              `<${it} ref="${it}" keep-alive style=%s></${it}>`,
              [this.current === it ? '' : 'display:none']
            ]
      )

      return raw(
        template.map(it => it[0]).join(''),
        template.map(it => it[1]).flat()
      )
    } else {
      if (this.current) {
        if (this.transition) {
          return raw(`<${this.current} #animation="%s"></${this.current}>`, [
            option
          ])
        }
        return raw(`<${this.current}></${this.current}>`)
      }
    }
  }
}

class RouterLink extends Component {
  static props = {
    to: Object,
    disabled: false
  }

  static styles = css`
    :host {
      display: inline-flex;
      align-items: center;
      -webkit-user-select: none;
      user-select: none;
    }
    a {
      display: flex;
      align-items: center;
      width: 100%;
      height: 100%;
      color: inherit;
      text-decoration: inherit;
      cursor: pointer;
    }

    :host([disabled]) a {
      opacity: 0.6;
      cursor: not-allowed;
    }
  `

  #href = ''

  #navigate() {
    let type = this.$router.type
    let { path } = this.to
    if (this.disabled) {
      return
    }

    if (type === 'hash') {
      location.hash = this.#href
    } else {
      this.$router.push(this.to)
    }
  }

  #parsePath() {
    let type = this.$router.type
    let { path = '', query = {} } = this.to
    let params =
      typeof query === 'string'
        ? query.replaceAll('?', '')
        : object2query(query)

    path = path.replace(/^\//, '')

    if (params) {
      path += '?' + params
    }

    return '/' + path
  }

  mounted() {
    this.$router.rsync(this, route => {
      this.classList.toggle('active', route.path === this.to.path)
    })
  }

  render() {
    this.#href = this.#parsePath()
    return html`<a title=${this.#href} @click=${this.#navigate}>
      <slot></slot
    ></a>`
  }
}

if (!customElements.get('router-view')) {
  customElements.define('router-view', RouterView)
}

if (!customElements.get('router-link')) {
  customElements.define('router-link', RouterLink)
}
