/**
 * {}
 * @author yutent<yutent.io@gmail.com>
 * @date 2023/08/10 11:57:38
 */

import { Component } from 'wkit'
import { __STORE__ } from './constants.js'

function observe(obj) {
  if (obj === null) {
    return obj
  }

  return new Proxy(obj, {
    get(target, key, receiver) {
      let value = Reflect.get(target, key, receiver)
      // 当访问的值是对象时，需要对这个对象也进行代理
      if (typeof value === 'object') {
        return observe(value)
      }
      return value
    },
    set(target, key, value, receiver) {
      Reflect.set(target, key, value, receiver)
      window.wkitd.broadcast()
      return true
    }
  })
}

export function createStore(obj = {}) {
  let defined = false

  return function () {
    Object.defineProperty(Component.prototype, '$store', {
      get() {
        return window.wkitd.get(__STORE__)
      },
      set(val) {
        if (defined) {
          return console.error(
            'Can not set readonly property $store of Component'
          )
        }
        window.wkitd.set(__STORE__, observe(val))
        defined = true
      },
      enumerable: false
    })
    Component.prototype.$store = obj
  }
}
